/*
Portions of this code related to parallax mapping derived from
http://www.gamedev.net/community/forums/topic.asp?topic_id=387447
*/

int		nMaxSamples			= 50;
int		nMinSamples			= 4;

float2 ParallaxMap(float3 vEye, float2 vCoord)
{
	float fBumpScale = 0.05f;

	float fDepth = HeightMapTexture.Sample( samLinear, vCoord ).rgb;
	float2 vHalfOffset = normalize(vEye) * (fDepth) * fBumpScale;

	fDepth = (fDepth + HeightMapTexture.Sample( samLinear, vCoord + vHalfOffset) ) * 0.5;
	vHalfOffset = normalize(vEye) * (fDepth) * fBumpScale;

	fDepth = (fDepth + HeightMapTexture.Sample( samLinear, vCoord + vHalfOffset) ) * 0.5;
	vHalfOffset = normalize(vEye) * (fDepth) * fBumpScale;
	
	return vCoord + vHalfOffset;	
}

float2 ParallaxOcclusionMap( float3 vEye, float2 texCoord, float3 vNormal )
{
	float fParallaxLimit = length(vEye.xy) / vEye.z;
	fParallaxLimit *= 0.03;
	
	float2 vOffset = normalize( vEye.xy );
	vOffset = vOffset * fParallaxLimit;

	float3 E = normalize( vEye );
	float3 N = normalize( vNormal );

	int nNumSamples = 50;//(int)lerp( nMinSamples, nMaxSamples, dot( E, N ) );
	float fStepSize = 1.0 / (float)nNumSamples;

	float2 dx, dy;
	dx = ddx( texCoord );
	dy = ddy( texCoord );

	float2 vOffsetStep = fStepSize * vOffset;
	float2 vCurrOffset = float2( 0, 0 );
	float2 vLastOffset = float2( 0, 0 );
	float2 vFinalOffset = float2( 0, 0 );

	float4 vCurrSample;
	float4 vLastSample;

	float stepHeight = 1.0;
	int nCurrSample = 0;

	while ( nCurrSample < nNumSamples )
	{
	   vCurrSample = HeightMapTexture.SampleGrad( samLinear, texCoord + vCurrOffset, dx, dy );
	   float fCurrSample = vCurrSample.rgb;
	   if ( fCurrSample > stepHeight )
	   {
		  float Ua = (vLastSample.rgb - (stepHeight+fStepSize))
					  / ( fStepSize + (vCurrSample.rgb - vLastSample.rgb ));
		  vFinalOffset = vLastOffset + Ua * vOffsetStep;

		  vCurrSample = HeightMapTexture.SampleGrad( samLinear, texCoord + vFinalOffset, dx, dy );
		  nCurrSample = nNumSamples + 1;
	   }
	   else
	   {
		  nCurrSample++;
		  stepHeight -= fStepSize;
		  vLastOffset = vCurrOffset;
		  vCurrOffset += vOffsetStep;
		  vLastSample = vCurrSample;
	   }
	}

	//return vCurrSample;
	//vFinalOffset = (vFinalOffset.x + dx, vFinalOffset.y + dy);
	return  texCoord + vFinalOffset;
}